
-- Controls des Config UI
local controls = {};

local DRAGON_STYLE = {
    "no dragon",
    "elite",
    "rare",
    "rare elite",
    "colored",
}

local ACTIONCAM_MODE = {
    "off",
    "default",
    "basic",
    "full",
}

local CUI_CONFIG_SHOWN = false;

function CleanUIConfig_OnLoad(self)
    CleanUIConfig_CreateUI(self);

    self:RegisterEvent("ADDON_LOADED");
    self:RegisterEvent("CVAR_UPDATE");

    hooksecurefunc(InterfaceOptionsFrame, "Show", CleanUIConfig_InterfaceOptionsFrame_Show);
    hooksecurefunc(InterfaceOptionsFrame, "Hide", CleanUIConfig_InterfaceOptionsFrame_Hide);

    cui_config_loaded = true;
    
    CleanUIConfig_ReloadUI();
end

function CleanUIConfig_InterfaceOptionsFrame_Show()
    if ( InterfaceOptionsFrame:IsShown() ) then
        CleanUIConfig_ReloadUI();
    end

    CUI_CONFIG_SHOWN = true;
end

function CleanUIConfig_InterfaceOptionsFrame_Hide()
    CUI_CONFIG_SHOWN = false;
end

function CleanUIConfig_OnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_BindingUI" ) then
            CleanUIConfig_ChangeBindingUI();
        end
    end
end

function CleanUIConfig_ReloadUI()
    if (not CleanUserData) then
        return;
    end

    UIDropDownMenu_SetSelectedValue(_G["CUIConfig_DragonStyleDropDown"], CleanUserData.dragon + 1);
    UIDropDownMenu_SetText(_G["CUIConfig_DragonStyleDropDown"], DRAGON_STYLE[CleanUserData.dragon + 1]);
    
    UIDropDownMenu_SetText(_G["CUIConfig_ActionCamModeDropDown"], CleanUISavedVariables.ActionCam);
    
    _G["CUIConfig_UnitFrameScale"]:SetValue(CleanUserData.scaleUnitFrames);
    _G["CUIConfig_PartyScale"]:SetValue(CleanUserData.scalePartyFrames);
    _G["CUIConfig_MiniMapScale"]:SetValue(CleanUserData.scaleMiniMap);
    _G["CUIConfig_ActionBarsScale"]:SetValue(CleanUserData.scaleActionBars);
        
    _G["CUIConfig_ToggleCursor"]:SetChecked(not CleanUISavedVariables.noCursorFlash);
    _G["CUIConfig_ToggleDigsiteButton"]:SetChecked(not CleanUISavedVariables.noDigsiteButton);
    _G["CUIConfig_ToggleGlobalCooldown"]:SetChecked(CleanUISavedVariables.CASTINGBARS_GLOBALCOOLDOWN);
    _G["CUIConfig_ToggleLatency"]:SetChecked(CleanUISavedVariables.CASTINGBARS_LATENCY);
end

function CleanUIConfig_CreateUI(self)

    -- Hauptfenster
    self.name="CleanUI";
    self.addonname = "CleanUI";
    InterfaceOptions_AddCategory(self);

    local author = GetAddOnMetadata("CleanUI", "Author");
    local version = GetAddOnMetadata("CleanUI", "Version");

    -- ConfigFrame
    CleanUIConfig_CreateHeader(self, "CleanUI v"..version , "("..author..")");
    
    CleanUIConfig_AddControl(self, CUIConfig_Reset);
    
    CleanUIConfig_AddLabel(self, "CUICommonHeader", "COMMON");

    -- Dragon

    CleanUIConfig_AddDropDown(self, "CUIConfig_DragonStyleDropDown",
        "Dragon style", CUIConfig_DragonStyleDropDown_Initialize);
    
    -- ActionCam

    CleanUIConfig_AddDropDown(self, "CUIConfig_ActionCamModeDropDown",
        "ActionCam mode", CUIConfig_ActionCamModeDropDown_Initialize);
    
    -- Scale
    CleanUIConfig_AddLabel(self, "CUI_ScaleHeader", "SCALE");
    
    CleanUIConfig_AddSlider(self, "CUIConfig_UnitFrameScale",
        "scale unit frames", 0.75, 2, 0.01, CleanUIConfig_ScaleUnitFrames);
    
    CleanUIConfig_AddSlider(self, "CUIConfig_PartyScale",
        "scale party unit frames", 0.75, 2, 0.01, CleanUIConfig_ScaleParty);
    
    CleanUIConfig_AddSlider(self, "CUIConfig_MiniMapScale",
        "scale mini map", 0.75, 2, 0.01, CleanUIConfig_ScaleMiniMap);
    
    CleanUIConfig_AddSlider(self, "CUIConfig_ActionBarsScale",
        "scale action buttons", 0.75, 2, 0.01, CleanUIConfig_ScaleActionBars);
    
    -- Enable/Disable
    CleanUIConfig_AddLabel(self, "CUI_EnDisHeader", "ENABLE/DISABLE");

    CleanUIConfig_AddCheckButton(self, "CUIConfig_ToggleCursor",
        "enable cursor flash", CleanUIConfig_ToggleCursorFlash);

    CleanUIConfig_AddCheckButton(self, "CUIConfig_ToggleDigsiteButton",
        "enable digsite button", CleanUIConfig_ToggleDigsiteButton);

    CleanUIConfig_AddCheckButton(self, "CUIConfig_ToggleGlobalCooldown",
        "enable global cooldown", CleanUIConfig_ToggleGlobalCooldown);

    CleanUIConfig_AddCheckButton(self, "CUIConfig_ToggleLatency",
        "enable latency bar", CleanUIConfig_ToggleLatency);
end

-- Dragon

function CUIConfig_DragonStyleDropDown_Initialize(self)
    for i = 1, #DRAGON_STYLE do
        local info = UIDropDownMenu_CreateInfo();
        info.text = DRAGON_STYLE[i];
        info.value = i;
        info.func = function()
            CleanUIConfig_SelectDragonStyle(i);
        end
        UIDropDownMenu_AddButton(info);
    end
end

function CleanUIConfig_SelectDragonStyle(i)
    CleanUI_SetUnitFrameDragon(i - 1);
    CleanUIConfig_ReloadUI();
end

-- ActionCam

function CUIConfig_ActionCamModeDropDown_Initialize(self)
    for i = 1, #ACTIONCAM_MODE do
        local info = UIDropDownMenu_CreateInfo();
        info.text = ACTIONCAM_MODE[i];
        info.value = i;
        info.func = function()
            CleanUIConfig_SelectActionCamMode(i);
        end
        UIDropDownMenu_AddButton(info);
    end
end

function CleanUIConfig_SelectActionCamMode(i)
    CleanUI_SetActionCamMode(i - 1);
    CleanUIConfig_ReloadUI();
end

-- Scale

function CleanUIConfig_ScaleUnitFrames(self)
    CleanUI_SetUnitFramesScale(self:GetValue());
end

function CleanUIConfig_ScaleParty(self)
    CleanUI_SetPartyScale(self:GetValue());
end

function CleanUIConfig_ScaleMiniMap(self)
    CleanUI_SetMiniMapScale(self:GetValue());
end

function CleanUIConfig_ScaleActionBars(self)
    CleanUI_SetActionBarsScale(self:GetValue());
end

-- Enable/Disable

function CleanUIConfig_ToggleCursorFlash()
    CleanUISavedVariables.noCursorFlash = not CleanUISavedVariables.noCursorFlash;
    CleanUIConfig_ReloadUI();
end

function CleanUIConfig_ToggleDigsiteButton()
    CleanUISavedVariables.noDigsiteButton = not CleanUISavedVariables.noDigsiteButton;
    CleanUIConfig_ReloadUI();
end

function CleanUIConfig_ToggleGlobalCooldown()
    CleanUISavedVariables.CASTINGBARS_GLOBALCOOLDOWN = not CleanUISavedVariables.CASTINGBARS_GLOBALCOOLDOWN;
    CleanUIConfig_ReloadUI();
end

function CleanUIConfig_ToggleLatency()
    CleanUISavedVariables.CASTINGBARS_LATENCY = not CleanUISavedVariables.CASTINGBARS_LATENCY;
    CleanUIConfig_ReloadUI();
end

-- UI Tools

function CleanUIConfig_CreateHeader(parent, title, info)
    local headline, subtext;

    headline = parent:CreateFontString(parent:GetName().."Title", "ARTWORK", "CUIOptionsTitleTemplate");
    headline:SetText("|c333399ff"..title.."|r     "..info);
    headline:ClearAllPoints();
    headline:SetPoint("TOPLEFT", parent, "TOPLEFT", 16, -16);
end

function CleanUIConfig_AddControl(parent, control, xoffset)

    control:SetParent(parent);

    local liste = controls[parent:GetName()];
    if (not liste) then
        liste = {};
        controls[parent:GetName()] = liste;
    end

    local size = table.getn(liste);

    if (xoffset) then
        control.xoffset = xoffset;
    end

    local ref = parent;
    local relative = "TOPLEFT";
    local x = 15 + (xoffset or 0);
    local y = -45;

    if (size > 0) then
        ref = liste[size];
        relative = "BOTTOMLEFT";
        x = (xoffset or 0) - (ref.xoffset or 0);
        y = -5;
    end

    table.insert(liste, control);

    control:ClearAllPoints();
    control:SetPoint("TOPLEFT", ref, relative, x, y);

    return control;
end

function CleanUIConfig_AddLabel(parent, name, text)
    local info = parent:CreateFontString(name, "ARTWORK");
    info:SetFontObject("GameFontNormal");
    info:SetWidth(450);
    info:SetHeight(30);
    info:SetJustifyH("LEFT");
    info:SetJustifyV("BOTTOM");
    info:SetText("|cffffffff"..text.."|r");

    return CleanUIConfig_AddControl(parent, info, 10);
end

function CleanUIConfig_AddCheckButton(parent, name, text, onClick, size, xoffset)
    local checkButton = CreateFrame("CheckButton", name, parent, "CUIOptionsCheckButtonTemplate");
    checkButton.text:SetText(text);

    checkButton:SetScript("OnClick", onClick);

    if (size) then
        checkButton:SetSize(size, size);
    end

    return CleanUIConfig_AddControl(parent, checkButton, xoffset);
end

function CleanUIConfig_AddSlider(parent, name, text, min, max, step, onValueChanged, xoffset)
    local slider = CreateFrame("Slider", name, parent, "CUIOptionsSliderTemplate");

    slider:SetScript("OnValueChanged", onValueChanged);

    slider:SetMinMaxValues(min, max);
    slider:SetValueStep(step);

    local label = _G[slider:GetName().."Text"];
    label:SetText(text);
    label:ClearAllPoints();
    label:SetPoint("LEFT", slider, "RIGHT", 15, 0);
    label:SetFontObject("GameFontNormal");

    local textLow = _G[slider:GetName().."Low"];
    local textHigh = _G[slider:GetName().."High"];
    textLow:SetText(min);
    textHigh:SetText(max);

    return CleanUIConfig_AddControl(parent, slider, xoffset);
end

function CleanUIConfig_AddDropDown(parent, name, text, onInitialize, width, xoffset)
    local dropdown = CreateFrame("Frame", name, parent, "CUIDropDownMenuTemplate");

    dropdown.label:SetText(text);

    UIDropDownMenu_Initialize(dropdown, onInitialize);
    UIDropDownMenu_JustifyText(dropdown, "LEFT");

    if (width) then
        UIDropDownMenu_SetWidth(dropdown, width);
    end

    return CleanUIConfig_AddControl(parent, dropdown, xoffset);
end
