
function CleanUI_InitMap()
    CleanUI_MapHookEvents();
    CleanUI_RegisterMapEvents();
    
    CleanUI_ChangeMap();
    CleanUI_UpdateQuestHeader();
end

function CleanUI_RegisterMapEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");
    frame:RegisterEvent("WORLD_MAP_UPDATE");
    frame:RegisterEvent("QUEST_LOG_UPDATE");

    frame:HookScript("OnEvent", CleanUI_MapOnEvent);
    frame:HookScript("OnUpdate", CleanUI_UpdateMapCoords);
end

function CleanUI_MapOnEvent(self, event, ...)

    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "..." ) then
        end
    end

    if (event == "WORLD_MAP_UPDATE") then
        CleanUI_UpdateMapCoords();
    end

    if (event == "QUEST_LOG_UPDATE") then
        CleanUI_UpdateQuestHeader();
    end
end

function CleanUI_UpdateQuestHeader()
    local numQuests = tostring(select(2, GetNumQuestLogEntries()));
    local text = TRACKER_HEADER_QUESTS.."    "..numQuests.."/"..MAX_QUESTS;
    ObjectiveTrackerBlocksFrame.QuestHeader.Text:SetText(text);
end

function CleanUI_MapHookEvents()
    hooksecurefunc("WorldMapFrame_OnUpdate", CleanUI_UpdateMapCoords);
    
    hooksecurefunc("WorldMap_AddQuestTimeToTooltip", function(questID)
        local timeLeftMinutes = C_TaskQuest.GetQuestTimeLeftMinutes(questID) * 60;
        if timeLeftMinutes then
            _G["WorldMapTooltipTextLeft"..WorldMapTooltip:NumLines()]:SetText(BONUS_OBJECTIVE_TIME_LEFT:format(SecondsToTime(timeLeftMinutes)));
        end
    end)
    
    hooksecurefunc("WorldMapFrame_AnimAlphaIn", CleanUI_WorldMapFrame_AnimAlphaIn);
    hooksecurefunc("WorldMapFrame_AnimAlphaOut", CleanUI_WorldMapFrame_AnimAlphaOut);
end

function CleanUI_WorldMapFrame_AnimAlphaIn(self, useStartDelay)
    WorldMapFrameNavBar:Show();
    WorldMapFrameTutorialButton:Show();
    WorldMapFrame.BorderFrame:Show();
    QuestScrollFrameScrollBar:Show();
end

function CleanUI_WorldMapFrame_AnimAlphaOut(self, useStartDelay)
    WorldMapFrameNavBar:Hide();
    WorldMapFrameTutorialButton:Hide();
    WorldMapFrame.BorderFrame:Hide();
    QuestScrollFrameScrollBar:Hide();
end

function CleanUI_ChangeMap()
    -- Koordinaten
    local coords = WorldMapFrame:CreateFontString("CUIWorldMapFrameCoords", "OVERLAY");
    coords:SetFontObject("GameFontNormalLarge");
    coords:SetWidth(WorldMapDetailFrame:GetWidth());
    coords:SetHeight(20);
    coords:ClearAllPoints();
    coords:SetPoint("BOTTOMLEFT", WorldMapFrame, "BOTTOMLEFT", 0, -20);
    coords:SetPoint("BOTTOMRIGHT", WorldMapFrame, "BOTTOMRIGHT", 0, -20);
    coords:SetJustifyH("CENTER");
    coords:SetText("Hallo");
end

function CleanUI_UpdateMapCoords()
    if (not WorldMapFrame:IsVisible()) then
        return;
    end

    local player_x, player_y = CleanUI_GetPlayerMapPosition();
    local cursor_x, cursor_y = CleanUI_GetCursorPosition();

    local output = "";

    if (cursor_x == nil or cursor_y == nil) then
        output = UnitName("player")..": "..player_x.."|cffffffff/|r"..player_y;
    else
        output = UnitName("player")..": "..player_x.."|cffffffff/|r"..player_y.." -- Cursor: "..cursor_x.."|cffffffff/|r"..cursor_y;
    end

    CUIWorldMapFrameCoords:SetText(output);
end

function CleanUI_GetPlayerMapPosition()
    local x, y = GetPlayerMapPosition("player");
    
    x = x or 0;
    y = y or 0;

    x = x * 100;
    y = y * 100;

    return "|cffffffff"..string.format("%.1f", x).."|r", "|cffffffff"..string.format("%.1f", y).."|r";
end

function CleanUI_GetCursorPosition()

    local scale = WorldMapDetailFrame:GetEffectiveScale();
    local width = WorldMapDetailFrame:GetWidth();
    local height = WorldMapDetailFrame:GetHeight();
    local centerX, centerY = WorldMapDetailFrame:GetCenter();

    local x, y = GetCursorPosition();

    local adjustedX = (x / scale - (centerX - (width/2))) / width;
    local adjustedY = (centerY + (height/2) - y / scale) / height;

    adjustedX = adjustedX * 100;
    adjustedY = adjustedY * 100;

    if (adjustedX > 100 or adjustedY > 100) then
        return nil, nil, 0, 0;
    end

    return "|cffffffff"..string.format("%.1f", adjustedX).."|r", "|cffffffff"..string.format("%.1f", adjustedY).."|r", adjustedX, adjustedY;
end