local BOTTOM_OFFSET = 275;

local RAID_FLARE_ICONS = {};
RAID_FLARE_ICONS[1] = "Interface\\AddOns\\CleanUI\\skins\\marker\\blue";
RAID_FLARE_ICONS[2] = "Interface\\AddOns\\CleanUI\\skins\\marker\\green";
RAID_FLARE_ICONS[3] = "Interface\\AddOns\\CleanUI\\skins\\marker\\purple";
RAID_FLARE_ICONS[4] = "Interface\\AddOns\\CleanUI\\skins\\marker\\red";
RAID_FLARE_ICONS[5] = "Interface\\AddOns\\CleanUI\\skins\\marker\\yellow";
RAID_FLARE_ICONS[6] = "Interface\\AddOns\\CleanUI\\skins\\marker\\orange";
RAID_FLARE_ICONS[7] = "Interface\\AddOns\\CleanUI\\skins\\marker\\silver";
RAID_FLARE_ICONS[8] = "Interface\\AddOns\\CleanUI\\skins\\marker\\white";

local microbutton_names = {
    "CharacterMicroButton",
    "SpellbookMicroButton",
    "TalentMicroButton",
    "AchievementMicroButton",
    "QuestLogMicroButton",

    "GuildMicroButton",
    "LFDMicroButton",
    "CollectionsMicroButton",
    "EJMicroButton",
    "StoreMicroButton",

    "MainMenuMicroButton",
    "HelpMicroButton",
}

local raidIconSnaped = false;
local raidFlareSnaped = false;

function CleanUI_InitMicroButtons()
    -- create micro button bar
    local buttonbar = CreateFrame("Frame", "CUI_TopFrame", UIParent);
    CleanUI_SetBackdrop(buttonbar);
    
    buttonbar:SetSize(300, 45);
    buttonbar:SetFrameStrata("BACKGROUND");
    
    buttonbar:SetPoint("RIGHT", UIParent, "BOTTOMLEFT", 5, BOTTOM_OFFSET);
    
    buttonbar:EnableMouse(true);
    buttonbar:HookScript("OnEnter", CleanUI_TopBar_OnEnter);
    buttonbar:HookScript("OnLeave", CleanUI_TopBar_OnLeave);
    
    -- create raid icon bar
    local raidbar = CreateFrame("Frame", "CUI_TopFrameRaidBar", UIParent);
    CleanUI_SetBackdrop(raidbar);
    
    raidbar:SetSize(169, 27);
    raidbar:SetFrameStrata("BACKGROUND");
    
    raidbar:ClearAllPoints();
    raidbar:SetPoint("BOTTOMLEFT", buttonbar, "TOPLEFT", 0, -4);
    
    raidbar:EnableMouse(true);
    raidbar:HookScript("OnEnter", CleanUI_TopBar_OnEnter);
    raidbar:HookScript("OnLeave", CleanUI_TopBar_OnLeave);
    
    -- create flare bar
    local flarebar = CreateFrame("Frame", "CUI_TopFrameFlareBar", UIParent);
    CleanUI_SetBackdrop(flarebar);
    
    flarebar:SetSize(135, 27);
    flarebar:SetFrameStrata("BACKGROUND");
    
    flarebar:ClearAllPoints();
    flarebar:SetPoint("BOTTOMRIGHT", buttonbar, "TOPRIGHT", 0, -4);
    
    flarebar:EnableMouse(true);
    flarebar:HookScript("OnEnter", CleanUI_TopBar_OnEnter);
    flarebar:HookScript("OnLeave", CleanUI_TopBar_OnLeave);

    hooksecurefunc("UpdateMicroButtons", CleanUI_RelocateMicroButtons);
    hooksecurefunc("MoveMicroButtons", CleanUI_RelocateMicroButtons);
    
    CleanUI_RelocateMicroButtons();
    
    SpellbookMicroButton:HookScript("OnEnter", CleanUI_MicroButton_OnEnter);
    hooksecurefunc("MicroButton_OnEnter", CleanUI_MicroButton_OnEnter);

    for ii=1, #(microbutton_names), 1 do
        local button = _G[microbutton_names[ii]];

        if (button) then
            button:HookScript("OnEnter", CleanUI_TopBar_OnEnter);
            button:HookScript("OnLeave", CleanUI_TopBar_OnLeave);

            button:SetFrameStrata("DIALOG");
        end
    end
    
    CleanUI_CreateRaidIconBar();
    CleanUI_CreateRaidFlareBar();
    
    CleanUI_ResetRaidIconPosition()
    CleanUI_ResetRaidFlarePosition();
end

function CleanUI_MicroButton_OnEnter(self)
    GameTooltip:ClearAllPoints();
    GameTooltip:SetPoint("TOP", self, "BOTTOM", 0, 0);
end

function CleanUI_TopBar_OnEnter()

    if (raidIconSnaped or raidFlareSnaped) then
        CleanUI_TopBar_OnLeave();
        return;
    end

    CUI_TopFrame:SetPoint("RIGHT", UIParent, "BOTTOMLEFT", CUI_TopFrame:GetWidth() - 5, BOTTOM_OFFSET);
    
    CUI_TopFrameRaidBar:ClearAllPoints();
    CUI_TopFrameRaidBar:SetPoint("BOTTOMLEFT", CUI_TopFrame, "TOPLEFT", 0, -4);
    
    CUI_TopFrameFlareBar:ClearAllPoints();
    CUI_TopFrameFlareBar:SetPoint("BOTTOMRIGHT", CUI_TopFrame, "TOPRIGHT", 0, -4);
end

function CleanUI_TopBar_OnLeave()
    CUI_TopFrame:SetPoint("RIGHT", UIParent, "BOTTOMLEFT", 5, BOTTOM_OFFSET);
    
    CUI_TopFrameRaidBar:ClearAllPoints();
    CUI_TopFrameRaidBar:SetPoint("BOTTOMLEFT", CUI_TopFrame, "TOPLEFT", 0, -4);
    
    CUI_TopFrameFlareBar:ClearAllPoints();
    CUI_TopFrameFlareBar:SetPoint("BOTTOMRIGHT", CUI_TopFrame, "TOPRIGHT", 0, -4);
end

function CleanUI_RelocateMicroButtons()
    CharacterMicroButton:ClearAllPoints();
    CharacterMicroButton:SetPoint("LEFT", _G["CUI_TopFrame"], "LEFT", 7, 10);

    for ii=2, #(microbutton_names), 1 do
        local button = _G[microbutton_names[ii]];
        local buttonref = _G[microbutton_names[ii-1]];

        if (button and buttonref) then
            button:ClearAllPoints();
            button:SetPoint("LEFT", buttonref, "RIGHT", -2, 0);
        end
    end
end

function CleanUI_CreateRaidIconBar()
    local raidicons = CreateFrame("Frame", "CUI_ChatRaidIcons", UIParent);
    raidicons:SetSize(160, 23);
    raidicons:SetScale(1);

    for i=1, 8, 1 do

        local posx = -6-(i-1) * 16;

        local btn = CreateFrame("Button", "CUI_CHAT_RAIDICON_"..i, raidicons, "UIPanelButtonTemplate");

        btn:SetNormalTexture(CUI_RAIDICON_FILE);
        btn:SetPushedTexture(CUI_RAIDICON_FILE);
        btn:GetNormalTexture():SetTexCoord(CUI_RAIDICONS[i].left, CUI_RAIDICONS[i].right, CUI_RAIDICONS[i].top, CUI_RAIDICONS[i].bottom);
        btn:GetPushedTexture():SetTexCoord(CUI_RAIDICONS[i].left, CUI_RAIDICONS[i].right, CUI_RAIDICONS[i].top, CUI_RAIDICONS[i].bottom);
        CleanUI_RemoveUiButtonTexture(btn);

        btn:RegisterForClicks("AnyUp");

        btn:SetWidth(13);
        btn:SetHeight(13);
        btn:ClearAllPoints();
        btn:SetPoint("RIGHT", raidicons, "RIGHT", posx, 0);

        btn:SetAttribute("index", i);

        btn:HookScript("OnClick", CleanUI_RaidIconClicked);
        btn:HookScript("OnEnter", CleanUI_TopBar_OnEnter);
        btn:HookScript("OnLeave", CleanUI_TopBar_OnLeave);
    end
    
    HelpOpenWebTicketButton:HookScript("OnEnter", CleanUI_TopBar_OnEnter);
    HelpOpenWebTicketButton:HookScript("OnLeave", CleanUI_TopBar_OnLeave);

    -- rc button
    local rcbutton = CreateFrame("Button", "CUI_CHAT_RCBUTTON", raidicons, "UIPanelButtonTemplate");

    rcbutton:SetNormalTexture("interface\\RaidFrame\\ReadyCheck-Ready");
    rcbutton:SetPushedTexture("interface\\RaidFrame\\ReadyCheck-Ready");
    rcbutton:SetHighlightTexture("interface\\RaidFrame\\ReadyCheck-Ready");
    CleanUI_RemoveUiButtonTexture(rcbutton);

    rcbutton:GetNormalTexture():SetTexCoord(0, 1, 0, 1);
    rcbutton:GetPushedTexture():SetTexCoord(0, 1, 0, 1);
    rcbutton:GetHighlightTexture():SetTexCoord(0, 1, 0, 1);

    rcbutton:SetWidth(18);
    rcbutton:SetHeight(18);
    rcbutton:ClearAllPoints();
    rcbutton:SetPoint("LEFT", raidicons, "LEFT", 3, 0);
    rcbutton:HookScript("OnClick", function() DoReadyCheck(); end);
    
    rcbutton:HookScript("OnEnter", CleanUI_TopBar_OnEnter);
    rcbutton:HookScript("OnLeave", CleanUI_TopBar_OnLeave);
    
    raidicons:SetParent(ChatFrame1);
end

function CleanUI_CreateRaidFlareBar()
    local raidflares = CreateFrame("Frame", "CUI_ChatRaidFlares", UIParent);
    raidflares:SetSize(130, 23);
    raidflares:SetScale(1);

    for i=1, 8, 1 do
        local posx = -6-(i-1) * 15;

        local btn = CreateFrame("Button", "CUI_CHAT_RAIDFLARE_"..i, raidflares, "SecureActionButtonTemplate");

        btn:SetNormalTexture(RAID_FLARE_ICONS[i]);
        btn:SetPushedTexture(RAID_FLARE_ICONS[i]);

        btn:RegisterForClicks("AnyUp");
        btn:SetAttribute("type1", "macro");
        btn:SetAttribute("macrotext1", "/worldmarker "..i);
        btn:SetAttribute("type2", "macro");
        btn:SetAttribute("macrotext2", "/clearworldmarker "..i);

        btn:SetWidth(13);
        btn:SetHeight(13);
        btn:ClearAllPoints();
        btn:SetPoint("RIGHT", raidflares, "RIGHT", posx, 0);

        btn:SetAttribute("index", i);
        
        btn:HookScript("OnEnter", CleanUI_TopBar_OnEnter);
        btn:HookScript("OnLeave", CleanUI_TopBar_OnLeave);
    end
    
    raidflares:SetParent(ChatFrame1);
end

function CleanUI_RaidIconSnap()
    local frame = _G["CUI_ChatRaidIcons"];

    icons_posx = frame:GetLeft();
    icons_posy = frame:GetBottom();

    local x, y = GetCursorPosition();
    local scale = UIParent:GetEffectiveScale();

    x = x / scale;
    y = y / scale;

    frame:ClearAllPoints();
    frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);
    
    raidIconSnaped = true;

    CleanUI_SetBackdrop(frame);
end

function CleanUI_ResetRaidIconPosition()
    local frame = _G["CUI_ChatRaidIcons"];
    local borderframe = _G["CUI_TopFrameRaidBar"];
    
    if (borderframe) then
        frame:ClearAllPoints();
        frame:SetPoint("CENTER", borderframe, "CENTER", 0, 0);   
    
        frame:SetBackdrop(nil);  
    else
        frame:ClearAllPoints();
        frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", 100000, 0);
    end
    
    raidIconSnaped = false;
end

function CleanUI_RaidIconClicked(self, button, down)

    local index = self:GetAttribute("index");

    if (not UnitExists("target") or IsAltKeyDown()) then
        CleanUI_ChatInsertText("{rt"..index.."}");
    elseif (button == "LeftButton") then
        SetRaidTarget("target", index);
    elseif (button == "RightButton") then
        SetRaidTarget("target", 0);
    end
end

function CleanUI_ChatInsertText(text)
    ChatMenu:Hide();

    if (not DEFAULT_CHAT_FRAME.editBox:IsShown()) then
        DEFAULT_CHAT_FRAME.editBox:Show();
    end

    DEFAULT_CHAT_FRAME.editBox:SetFocus()
    DEFAULT_CHAT_FRAME.editBox:Insert(text);
end

function CleanUI_RaidFlareSnap()
    if (InCombatLockdown()) then
        return;
    end

    local doSnap = true;

    if (IsInRaid()) then
        if ( UnitIsGroupLeader("player") or UnitIsGroupAssistant("player") ) then
            doSnap = true;
        else
            doSnap = false;
        end
    end

    if (not doSnap) then
        return;
    end

    local frame = _G["CUI_ChatRaidFlares"];

    icons_posx = frame:GetLeft();
    icons_posy = frame:GetBottom();

    local x, y = GetCursorPosition();
    local scale = UIParent:GetEffectiveScale();

    x = x / scale;
    y = y / scale;

    frame:ClearAllPoints();
    frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);
    
    raidFlareSnaped = true;

    CleanUI_SetBackdrop(frame);
end

function CleanUI_ResetRaidFlarePosition()
    if (InCombatLockdown()) then
        return;
    end
    
    local frame = _G["CUI_ChatRaidFlares"];
    local borderframe = _G["CUI_TopFrameFlareBar"];
    
    if (borderframe) then
        frame:ClearAllPoints();
        frame:SetPoint("CENTER", borderframe, "CENTER", 0, 0);   
    
        frame:SetBackdrop(nil); 
    else
        frame:ClearAllPoints();
        frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", 100000, 0);
    end
    
    raidFlareSnaped = false;
end